

#include "debug.H"
#include "eth_raw.H"

#include <iostream>
#include <iomanip>


using namespace std;
using namespace mhp;


TEthRaw::TEthRaw(TSinc &sinc, const string &deviceName, TEthRawListener *listener, unsigned char *allowedSourceMAC) {
	this->sinc = &sinc;
	this->setDeviceName(deviceName);
	this->setListener(listener);
	this->setAllowedSourceMAC(allowedSourceMAC);
	this->sendP = NULL;
	this->s = -1;
}


TEthRaw::~TEthRaw() {
}


void TEthRaw::showMAC(ostream &c, unsigned char *mac) {
	int i;
	for (i = 0; i < ETH_ALEN; i++) {
		c << hex << setfill('0') << setw(2) << (unsigned) mac[i];
		if (i < 5)
			c << ":";
	}
	c << dec << setfill(' ') << setw(0);
}


void TEthRaw::setDeviceName(const string &deviceName) {
	this->deviceName.assign(deviceName);
}


void TEthRaw::setListener(TEthRawListener *listener) {
	this->listener = listener;
}


void TEthRaw::setAllowedSourceMAC(unsigned char *mac) {
	if (mac != NULL) {
		memcpy(this->allowedSourceMAC, mac, ETH_ALEN);
		this->filter = true;
	}
	else
		this->filter = false;
}


bool TEthRaw::connect() {
	cout << "TEthRaw.connect: creating raw ethernet non-blocking socket... ";
	if ((this->s = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
		cout << "ERROR" << endl;
		cerr << "TEthRaw.connect: ERROR at socket " << strerror(errno) << endl;
		return false;
	}
	this->sinc->setRead(this->s);
	if (fcntl(this->s, F_SETFL, O_NONBLOCK) == -1) {
		cout << "ERROR" << endl;
		cerr << "TEthRaw.connect: ERROR at fcntl " << strerror(errno) << endl;
		return false;
	}
	cerr << "ok" << endl;
	/* obtenemos el ndice del interface */
	cerr << "TEthRaw.connect: searching device " << this->deviceName << "... ";
	strncpy(this->ifr.ifr_name, this->deviceName.c_str(), IFNAMSIZ);
	if (ioctl(this->s, SIOCGIFINDEX, &this->ifr) == -1) {
		cerr << "ERROR" << endl;
		cerr << "TEthRaw.connect: ERROR at ioctl SIOCGIFINDEX " << strerror(errno) << endl;
		return false;
	}
	this->ifindex = this->ifr.ifr_ifindex;
	cerr << "found with index " << this->ifindex << endl;
	/* obtenemos la direccin hardware */
	cerr << "TEthRaw.connect: getting hardware address for device " << this->deviceName << "... ";
	strncpy(this->ifr.ifr_name, this->deviceName.c_str(), IFNAMSIZ);
	if (ioctl(this->s, SIOCGIFHWADDR, &this->ifr) == -1) {
		cerr << "ERROR" << endl;
		cerr << "TEthRaw.connect: ERROR at ioctl SIOCGIFHWADDR " << strerror(errno) << endl;
		return false;
	}
	this->showMAC(cerr, (unsigned char *) this->ifr.ifr_hwaddr.sa_data);
	cerr << endl;
	memcpy(this->deviceMAC, this->ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	this->recvP = this->recvBuffer;
	this->remainRecvSize = ETH_FRAME_LEN;
	this->sendP = NULL;
	return true;
}


void TEthRaw::send(unsigned char *destMAC, void *data, int size) {
	if (this->s < 0)
		return;
	size = (size > 1500) ? 1500 : size;
	this->ether_head = this->sendBuffer;
	this->data = this->sendBuffer + 14;
	this->eh = (struct ethhdr *) ether_head;
	this->sa.sll_family = PF_PACKET;
	// this->sa.sll_protocol = htons(ETH_P_IP);
	this->sa.sll_protocol = htons(size);
	this->sa.sll_ifindex = this->ifindex;                   /* interface index */
	this->sa.sll_hatype = ARPHRD_ETHER;
	this->sa.sll_pkttype = PACKET_OTHERHOST;
	this->sa.sll_halen = ETH_ALEN;              /* tamao de la direccin ethernet */
	bzero(this->sa.sll_addr, 8);
	memcpy(this->sa.sll_addr, destMAC, ETH_ALEN);
	/* el paquete ethernet */
	memcpy(this->sendBuffer, destMAC, ETH_ALEN);
	memcpy(this->sendBuffer + ETH_ALEN, this->deviceMAC, ETH_ALEN);
	// send data
	eh->h_proto = htons(size);
	bzero(this->data, size);
	memcpy(this->data, data, size);
	/* enviamos el paquete */
	this->sendP = this->sendBuffer;
	//this->remainSendSize = ETH_FRAME_LEN;
	this->remainSendSize = 14 + size;
	/*
	int sendResult = sendto(this->s, this->sendBuffer, ETH_FRAME_LEN, 0, (struct sockaddr *) &this->sa, sizeof(this->sa));
	if (sendResult < 0) {
		cerr << "TEthRaw.connect: ERROR at sendto " << strerror(errno) << endl;
		return false;
	}
	cerr << "TEthRaw.send: raw ethernet packet sent (sendResult = " << sendResult << ")" << endl;
	*/
	this->sinc->setWrite(this->s);
}


int TEthRaw::run() {
	int ret = 0;
	if (this->s < 0)
		return ret;
	// check recv
	int length = recvfrom(this->s, this->recvP, this->remainRecvSize, 0, NULL, NULL);
	if (length > 0) {
		this->recvP += length;
		this->remainRecvSize -= length;
		//if (this->remainRecvSize <= 0) {
			memcpy(this->destMAC, this->recvBuffer, ETH_ALEN);
			memcpy(this->sourceMAC, this->recvBuffer + ETH_ALEN, ETH_ALEN);
			int i = ntohs(*((unsigned short int *) (this->recvBuffer + ETH_ALEN + ETH_ALEN)));
			if (((!this->filter) || (this->filter && (memcmp(this->sourceMAC, this->allowedSourceMAC, ETH_ALEN) == 0))) && (i < 0x0800)) {
				if (this->listener != NULL)
					this->listener->ethRawPacketReceived((void *) (this->recvBuffer + ETH_ALEN + ETH_ALEN + 2), i);
#ifdef DEBUG_ETH
				cerr << "TEthRaw.run: packet from ";
				this->showMAC(cerr, this->sourceMAC);
				cerr << " to ";
				this->showMAC(cerr, this->destMAC);
				cerr << " " << i << " bytes" << endl;
#endif
			}
			this->recvP = this->recvBuffer;
			this->remainRecvSize = ETH_FRAME_LEN;
			ret++;
		//}
	}
	// check send
	if (this->sendP != NULL) {
		length = sendto(this->s, this->sendP, this->remainSendSize, 0, (struct sockaddr *) &this->sa, sizeof(this->sa));
		if (length > 0) {
			this->sendP += length;
			this->remainSendSize -= length;
			if (this->remainSendSize <= 0) {
				this->sendP = NULL;
				if (this->listener != NULL)
					this->listener->ethRawPacketSent((void *) this->data);
#ifdef DEBUG_ETH
				cerr << "TEthRaw.run: packet sent" << endl;
#endif
				this->sinc->clrWrite(this->s);
			}
			ret++;
		}
	}
	return ret;
}


void TEthRaw::disconnect() {
	this->sinc->clrRead(this->s);
	this->sinc->clrWrite(this->s);
	close(this->s);
	this->s = -1;
}


