

#include "debug.H"
#include "pty.H"

#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <termios.h>

#include <string>
#include <iostream>


using namespace std;
using namespace mhp;


TPty::TPty(TSinc &sinc, TPtyListener *listener) {
	this->sinc = &sinc;
	this->setListener(listener);
	this->masterFd = -1;
	this->slaveName.clear();
}


TPty::~TPty() {
}


void TPty::setListener(TPtyListener *listener) {
	this->listener = listener;
}


bool TPty::open() {
	cerr << "TPty.open: opening virtual terminal... ";
	this->masterFd = ::open("/dev/ptmx", O_RDWR | O_NOCTTY);
	if (this->masterFd < 0) {
		cerr << "ERROR" << endl;
		cerr << "TPty.open: ERROR at open /dev/ptmx: " << strerror(errno) << endl;
		return false;
	}
	this->slaveName.assign(ptsname(this->masterFd));
	if (grantpt(this->masterFd) != 0) {
		cerr << "ERROR" << endl;
		cerr << "TPty.open: ERROR at grantpt: " << strerror(errno) << endl;
		return false;
	}
	if (unlockpt(this->masterFd) != 0) {
		cerr << "ERROR" << endl;
		cerr << "TPty.open: ERROR at unlockpt: " << strerror(errno) << endl;
		return false;
	}
	if (fcntl(this->masterFd, F_SETFL, O_NONBLOCK) == -1) {
		cerr << "ERROR" << endl;
		cerr << "TPty.open: ERROR at fcntl: " << strerror(errno) << endl;
		return false;
	}
	cerr << "ok" << endl;
	struct termios tio;
	tcgetattr(this->masterFd, &tio);
	cfmakeraw(&tio);
	tcsetattr(this->masterFd, TCSANOW, &tio);
	this->sinc->setRead(this->masterFd);
	this->sendP = NULL;
	return true;
}


const string &TPty::getSlaveName() {
	return this->slaveName;
}


void TPty::close() {
	this->sinc->clrRead(this->masterFd);
	this->sinc->clrWrite(this->masterFd);
	::close(this->masterFd);
	this->masterFd = -1;
	this->slaveName.clear();
}


void TPty::send(void *data, int size) {
	if (this->masterFd < 0)
		return;
	size = (size > PTY_BUFFER_SIZE) ? PTY_BUFFER_SIZE : size;
	memcpy(this->sendBuffer, data, size);
	this->sendP = this->sendBuffer;
	this->remainSendSize = size;
	this->sinc->setWrite(this->masterFd);
}


int TPty::run() {
	int ret = 0;
	if (this->masterFd < 0)
		return ret;
	// check recv
	int length = read(this->masterFd, this->recvBuffer, PTY_BUFFER_SIZE);
	if (length > 0) {
#ifdef DEBUG_PTY
		cerr << "TPty.run: data received from pty (length " << length << " bytes)" << endl;
#endif
		if (this->listener != NULL)
			this->listener->ptyDataReceived(this->recvBuffer, length);
		ret++;
	}
	// check send
	if (this->sendP != NULL) {
		length = write(this->masterFd, this->sendP, this->remainSendSize);
#ifdef DEBUG_PTY
		cerr << "TPty.run: send length=" << length << endl;
#endif
		if (length > 0) {
			this->sendP += length;
			this->remainSendSize -= length;
			if (this->remainSendSize <= 0) {
				this->sendP = NULL;
				if (this->listener != NULL)
					this->listener->ptyDataSent(this->sendBuffer);
			}
			ret++;
		}
	}
	return ret;
}


