
#ifndef  __SINC_H__
#define  __SINC_H__


#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <sys/time.h>


extern "C++" {
	namespace mhp {
		class TSinc {
			public:
				// Contructor
				TSinc();
				// Inicializa
				void clear();
				// Indica la espera en segundos
				void setSec(const int lossegundos);
				// Indica la espera en milisegundos
				void setUsec(const int losusegundos);
				// Espera a que algo ocurra y devuelve el descriptor (-1 error o timeout)
				int wait();
				// No espera a que algo ocurra y devuelve el descriptor (-1 error o timeout)
				int noWait();
				// Asocia el descriptor a los eventos de lectura
				void setRead(const int elfd);
				// Asocia el descriptor a los eventos de escritura
				void setWrite(const int elfd);
				// Asocia el descriptor a los eventos de excepciones
				void setExcept(const int elfd);
				// Desliga el descriptor a los eventos de lectura
				void clrRead(const int elfd);
				// Desliga el descriptor a los eventos de escritura
				void clrWrite(const int elfd);
				// Desliga el descriptor a los eventos de excepciones
				void clrExcept(const int elfd);
				// Verifica si el evento del descriptor ha sido por lectura
				bool isRead(const int elfd);
				// Verifica si el evento del descriptor ha sido por escritura
				bool isWrite(const int elfd);
				// Verifica si el evento del descriptor ha sido por excepcin
				bool isExcept(const int elfd);
			private:
				fd_set rfds, wfds, efds;
				fd_set xrfds, xwfds, xefds;
				int sec, usec;
				struct timeval tv;
				int event, max;
		};
	}
}

#endif  // __SINC_H__
