

#include "debug.H"
#include "soe.H"
#include <ctype.h>
#include <iostream>
#include <fstream>


using namespace std;
using namespace mhp;


TSoe::TSoe(const string &deviceName, unsigned char *otherSideMAC) : pty(sinc), ethRaw(sinc) {
	this->ethRaw.setDeviceName(deviceName);
	this->ethRaw.setListener(this);
	memcpy(this->otherSideMAC, otherSideMAC, ETH_ALEN);
	this->ethRaw.setAllowedSourceMAC(this->otherSideMAC);
	this->pty.setListener(this);
	this->sinc.setSec(1);
}


TSoe::~TSoe() {
}


string TSoe::getStringFromBuffer(void *data, int size) {
	string ret = "";
	unsigned char *p = (unsigned char *) data;
	for (int i; i < size; i++, p++) {
		if (isgraph(*p))
			ret += *p;
		else
			ret += '.';
	}
	return ret;
}


void TSoe::ethRawPacketReceived(void *data, int size) {
	// cout << "ETH DATA RECEIVED: '" << ((char *) data) << "'" << endl;
#ifdef DEBUG_SOE
	cout << "ETH DATA RECEIVED: '" << this->getStringFromBuffer(data, size) << "'" << endl;
#endif
	this->pty.send(data, size);
}


void TSoe::ethRawPacketSent(void *data) {
}


void TSoe::ptyDataReceived(void *data, int size) {
#ifdef DEBUG_SOE
	cout << "PTY DATA RECEIVED: '" << this->getStringFromBuffer(data, size) << "'" << endl;
#endif
	this->ethRaw.send(this->otherSideMAC, data, size);
}


void TSoe::ptyDataSent(void *data) {
}


int TSoe::run() {
	return this->pty.run() + this->ethRaw.run();
}


void TSoe::savePtsPath(const string &path) {
	ofstream os;
	os.open(path.c_str());
	os << this->pty.getSlaveName() << endl;
	os.close();
	cout << "pts path saved at " << path << endl;
}


void TSoe::mainLoop() {
	this->pty.open();
	// cout << "pts = '" << pty.getSlaveName() << "' (MTU=" << PTY_BUFFER_SIZE << ")" << endl;
	cout << "pts " << pty.getSlaveName() << endl;
	this->savePtsPath("/var/run/soepts");
	this->ethRaw.connect();
	while (true) {
		//cerr << "." << flush;
		this->sinc.wait();
		while (this->run() != 0)
			;
	}
}

/*
int main(int argc, char *argv[]) {
	if (argc < 3) {
		cerr << "Sintax: " << argv[0] << " ethernet_device_name other_end_mac" << endl;
		return 1;
	}
	unsigned char otherSideMAC[ETH_ALEN];
	sscanf(argv[2], "%x:%x:%x:%x:%x:%x", &otherSideMAC[0], &otherSideMAC[1], &otherSideMAC[2], &otherSideMAC[3], &otherSideMAC[4], &otherSideMAC[5]);
	TSoe soe(argv[1], otherSideMAC);
	soe.mainLoop();
}
*/
